@@ -6,7 +6,17 @@ import urlparse
 import lxml.html
 import sha
 import mimetypes
+import logging
 import webob
+import urllib
+
+logger = logging.getLogger("squeeze")
+    
+try:
+    from repoze.xmliter import XMLSerializer
+except ImportError:
+    def XMLSerializer(tree, serializer=None):
+        return lxml.html.tostring(tree, pretty_print=True)
 
 from cStringIO import StringIO
 
@@ -36,7 +46,7 @@ class ResourceSqueezingMiddleware(object):
         self.threshold = threshold
         self.accept_request_registry = {}
         
-    def get_merged_resource(self, cache, selection):
+    def get_merged_resource(self, cache, selection, mediatypes):
         expires = None
         
         # verify that the entire selection is in the cache
@@ -52,11 +62,15 @@ class ResourceSqueezingMiddleware(object):
 
         # compute digest
         out = StringIO()
-        for body, mimetype, ttl in map(cache.get, selection):
+        for url, body, mimetype, ttl in ((s,) + cache.get(s)
+                                         for s in selection):
+            mediatype = mediatypes.get(url)
             if expires is None or ttl < expires:
                 expires = ttl
-                
-            out.write(body)
+            if mimetype == 'text/css' and mediatype:
+                out.write('@media %s {\n%s}\n' % (mediatype, body))
+            else:
+                out.write(body)
         body = out.getvalue()
         digest = sha.new(body).hexdigest() + (ext or "")
 
@@ -103,17 +117,27 @@ class ResourceSqueezingMiddleware(object):
         # if url matches a URL we've seen in a processed document, and
         # if it's served from this host, process the response body and
         # cache it
-        if request.url in accept_request_data.appearances:
+        url = urllib.unquote(request.url)
+        if url in accept_request_data.appearances:
             ttl = response.expires
             if content_type == 'text/css':
-                base_path = os.path.dirname(request.url)
+                base_path = os.path.dirname(url)
                 response.body = re_stylesheet_url.sub(
                     'url(%s/\\1)' % base_path, response.body)
-                
-            accept_request_data.cache[request.url] = response.body, content_type, ttl
+
+            status_code = int(response.status.split(' ', 1)[0])
+            if status_code == 200:
+                accept_request_data.cache[url] = response.body, content_type, ttl
+            elif status_code == 304:
+                # update ttl for cached resource
+                cache = accept_request_data.cache.get(url)
+                if cache is not None:
+                    body, content_type, previous_ttl = cache
+                    accept_request_data.cache[url] = body, content_type, ttl
 
         return response(environ, start_response)
 
+
     def process_html(self, accept_request_data, host, uri, body):
         javascripts = accept_request_data.javascripts
         stylesheets = accept_request_data.stylesheets
@@ -135,7 +159,9 @@ class ResourceSqueezingMiddleware(object):
             elements = tree.xpath(xpath)
             for element in elements:
                 mutator, accessor = tag_functions[element.tag]
-                items.append(get_url(tree, host, uri, accessor(element)))
+                url = accessor(element)
+                if url is not None:
+                    items.append(get_url(tree, host, uri, url))
             selections = maintain_appearances(
                 items, groups, appearances, self.threshold)
             ttl, changed = self.update_elements(
@@ -144,30 +170,37 @@ class ResourceSqueezingMiddleware(object):
             if expires is None or ttl < expires:
                 expires = ttl                
                 
-        return changed, expires, lxml.html.tostring(tree, pretty_print=True)        
+        return changed, expires, XMLSerializer(tree, lxml.html.tostring)
 
     def update_elements(self, elements, selections, tree, host, uri, cache):
         changed = False
         expires = None
-        
-        for element in elements:
-            mutator, accessor = tag_functions[element.tag]
+        mediatypes = {}
 
+        for element in elements:
+            parent = element.getparent()
+            mutator, accessor = tag_functions[element.tag]
+            url = accessor(element)
+            if url is None:
+                continue
+            
             # prepend base path to relative path
-            src = get_url(tree, host, uri, accessor(element))
+            src = get_url(tree, host, uri, url)
+            mediatypes[src] = element.attrib.get('media')
 
             if not src.startswith(host):
                 continue
-            
+
             for selection in selections:
-                ttl, resource = self.get_merged_resource(cache, selection)
-                
+                ttl, resource = self.get_merged_resource(cache, selection,
+                                                         mediatypes)
+
                 if resource is None or src not in selection:
                     continue
 
                 if expires is None or ttl < expires:
                     expires = ttl
-                
+
                 # if this is the last item in the selection,
                 # merge and update link, else remove it.
                 if src == selection[-1]:
@@ -175,12 +208,11 @@ class ResourceSqueezingMiddleware(object):
                     changed = True
                     mutator(element, url)
                 else:
-                    parent = element.getparent()
-                    index = parent.index(element)
                     parent.remove(element)
-
+                    break
+                    
         return expires, changed
-    
+
 def update_script_tag(element, url):
     element.attrib['src'] = url
 
